# Makefile to build D runtime library ares.lib for Win32
# Designed to work with \dm\bin\make.exe
# Targets:
#	make
#		Same as make unittest
#	make ares.lib
#		Build ares.lib
#	make clean
#		Delete unneeded files created by build process
#	make unittest
#		Build ares.lib, build and run unit tests
# Notes:
#	This relies on LIB.EXE 8.00 or later, and MAKE.EXE 5.01 or later.

CP=copy /y

CFLAGS=-mn -6 -r -Igc
#CFLAGS=-g -mn -6 -r -Igc

DFLAGS=-release -O -inline
#DFLAGS=-g -release

TFLAGS=-O -inline
#TFLAGS=-g

CC=\bin\dm\bin\dmc
DC=\bin\dmd\bin\dmd

DOC=..\..\doc\ares

.DEFAULT: .asm .c .cpp .d .html .obj

.asm.obj:
	$(CC) -c $*

.c.obj:
	$(CC) -c $(CFLAGS) $* -of$*.obj

.cpp.obj:
	$(CC) -c $(CFLAGS) $* -of$*.obj

.d.obj:
	$(DC) -c $(DFLAGS) $* -of$*.obj

.d.html:
     $(DC) -c -o- -version=DDoc -Df$(DOC)\$*.html ares.ddoc $*.d

targets : lib doc
ares    : lib
lib     : ares.lib
doc     : ares.doc

######################################################

OBJ_STD= \
    std\atomic.obj \
    std\exception.obj \
    std\intrinsic.obj \
    std\memory.obj \
    std\regexp.obj \
    std\stdarg.obj \
    std\thread.obj

OBJ_STD_C= \
    std\c\complex.obj \
    std\c\ctype.obj \
    std\c\errno.obj \
    std\c\fenv.obj \
    std\c\inttypes.obj \
    std\c\math.obj \
    std\c\signal.obj \
    std\c\stdarg.obj \
    std\c\stdbool.obj \
    std\c\stddef.obj \
    std\c\stdint.obj \
    std\c\stdio.obj \
    std\c\stdlib.obj \
    std\c\string.obj \
    std\c\time.obj \
    std\c\wctype.obj

OBJ_STD_IO= \
    std\io\ArrayAllocator.obj \
    std\io\Buffer.obj \
    std\io\Conduit.obj \
    std\io\Console.obj \
    std\io\DeviceConduit.obj \
    std\io\Exception.obj \
    std\io\FileConduit.obj \
    std\io\FileConst.obj \
    std\io\FilePath.obj \
    std\io\FileProxy.obj \
    std\io\Reader.obj \
    std\io\Writer.obj

OBJ_STD_IO_CONVERT= \
    std\io\convert\Type.obj \
    std\io\convert\Unicode.obj

OBJ_STD_IO_MODEL= \
    std\io\model\IBuffer.obj \
    std\io\model\IConduit.obj \
    std\io\model\IReader.obj \
    std\io\model\IWriter.obj

OBJ_STD_IO_SYS= \
    std\io\sys\OS.obj

OBJ_STD_IO_UTILS= \
    std\io\utils\HeapSlice.obj

OBJ_STD_MATH= \
    std\math\core.obj \
    std\math\ieee.obj \
    std\math\special.obj

OBJ_SYS_WIN_C= \
    sys\windows\c\com.obj \
    sys\windows\c\process.obj \
    sys\windows\c\windows.obj

######################################################

DOC_STD= \
    $(DOC)\std\atomic.html \
    $(DOC)\std\exception.html \
    $(DOC)\std\intrinsic.html \
    $(DOC)\std\memory.html \
    $(DOC)\std\stdarg.html \
    $(DOC)\std\thread.html

######################################################

ares.lib : \
    $(OBJ_STD) \
    $(OBJ_STD_C) \
    $(OBJ_STD_IO)\
    $(OBJ_STD_IO_CONVERT) \
    $(OBJ_STD_IO_MODEL) \
    $(OBJ_STD_IO_SYS) \
    $(OBJ_STD_IO_UTILS) \
    $(OBJ_STD_MATH) \
    $(OBJ_SYS_WIN_C)
    del ares.lib
	lib -c -n ares.lib $(OBJ_STD) \
	                   $(OBJ_STD_C) \
                       $(OBJ_STD_IO)\
                       $(OBJ_STD_IO_CONVERT) \
                       $(OBJ_STD_IO_MODEL) \
                       $(OBJ_STD_IO_SYS) \
                       $(OBJ_STD_IO_UTILS) \
                       $(OBJ_STD_MATH) \
	                   $(OBJ_SYS_WIN_C)

ares.doc : \
    $(DOC_STD)

######################################################

### std

std\atomic.obj : std\atomic.d
    $(DC) -c $(TFLAGS) std\atomic.d  -odstd

std\stdarg.obj : std\stdarg.d
	$(DC) -c $(TFLAGS) std\stdarg.d  -odstd

### std\c

std\c\stdarg.obj : std\c\stdarg.d
	$(DC) -c $(TFLAGS) std\c\stdarg.d -odstd\c

######################################################

$(DOC)\std\atomic.html : ares.ddoc std\atomic.d
    $(DC) -c -o- -version=DDoc -Df$(DOC)\std\atomic.html ares.ddoc std\atomic.d

$(DOC)\std\exception.html : ares.ddoc std\exception.d
    $(DC) -c -o- -version=DDoc -Df$(DOC)\std\exception.html ares.ddoc std\exception.d

$(DOC)\std\intrinsic.html : ares.ddoc std\intrinsic.d
    $(DC) -c -o- -version=DDoc -Df$(DOC)\std\intrinsic.html ares.ddoc std\intrinsic.d

$(DOC)\std\memory.html : ares.ddoc std\memory.d
	$(DC) -c -o- -version=DDoc -Df$(DOC)\std\memory.html ares.ddoc std\memory.d

$(DOC)\std\stdarg.html : ares.ddoc std\stdarg.d
	$(DC) -c -o- -version=DDoc -Df$(DOC)\std\stdarg.html ares.ddoc std\stdarg.d

$(DOC)\std\thread.html : ares.ddoc std\thread.d
	$(DC) -c -o- -version=DDoc -Df$(DOC)\std\thread.html ares.ddoc std\thread.d

######################################################

clean :
	del $(OBJ_STD) \
        $(OBJ_STD_C) \
        $(OBJ_STD_IO)\
        $(OBJ_STD_IO_CONVERT) \
        $(OBJ_STD_IO_MODEL) \
        $(OBJ_STD_IO_SYS) \
        $(OBJ_STD_IO_UTILS) \
        $(OBJ_STD_MATH) \
        $(OBJ_SYS_WIN_C)
	del ares.lib
	del $(DOC_STD)

install :
	$(CP) ares*.lib ..\..\lib\.
